@@ -1,3 +1,6 @@
+{-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE Safe #-}
+
 -- |
 -- Module      :  Text.Megaparsec.Common
 -- Copyright   :  © 2018–present Megaparsec contributors
@@ -11,32 +14,29 @@
 -- it are re-exported in "Text.Megaparsec.Byte" and "Text.Megaparsec.Char".
 --
 -- @since 7.0.0
-
-{-# LANGUAGE FlexibleContexts #-}
-
 module Text.Megaparsec.Common
-  ( string
-  , string' )
+  ( string,
+    string',
+  )
 where
 
+import qualified Data.CaseInsensitive as CI
 import Data.Function (on)
 import Text.Megaparsec
-import qualified Data.CaseInsensitive as CI
 
 -- | A synonym for 'chunk'.
-
 string :: MonadParsec e s m => Tokens s -> m (Tokens s)
 string = chunk
 {-# INLINE string #-}
 
 -- | The same as 'string', but case-insensitive. On success returns string
--- cased as actually parsed input.
+-- cased as the parsed input.
 --
 -- >>> parseTest (string' "foobar") "foObAr"
 -- "foObAr"
-
-string' :: (MonadParsec e s m, CI.FoldCase (Tokens s))
-  => Tokens s
-  -> m (Tokens s)
+string' ::
+  (MonadParsec e s m, CI.FoldCase (Tokens s)) =>
+  Tokens s ->
+  m (Tokens s)
 string' = tokens ((==) `on` CI.mk)
 {-# INLINE string' #-}
